# -*- coding: utf-8 -*-
import connect as os_connect
from connect import OsDebug

class OsWalkers(object):
    currentWalker = False

    @staticmethod
    def connectWalker(walker):
        if OsWalkers.currentWalker:
            OsWalkers.currentWalker.disconnect()
        OsWalkers.currentWalker = walker

    @staticmethod
    def makePathToFile(walker, tofile):
        dirs = tofile.split('/')
        k = 1
        pathes = []
        for dir in dirs:
            if k < len(dirs) and len(dir) > 0:
                pathes += [ dir ]
                walker.doCommand('mkdir /' + '/'.join(pathes))
            k += 1

def ifHasElseFalse(mass, name):
    if name in mass:
        return mass[name]
    return False

def WidedDict(dict, wider):
    for w in wider.keys():
        dict[w] = wider[w]
    return dict
    '''
    return {
        'maybe': {
            'password:':self.password,
            'пароль:':self.password,
            }
    }
    '''

class OsWalker(object):
    def __init__(self, command=False, baseWalker=False, connectExpecter=False, connectOnInit=True):
        self.mayDebug = True
        self.command = command
        self.childWalker = False
        self.baseWalker = baseWalker
        self.connectExpecter_ = connectExpecter
        self.lastCommand = ''
        if self.baseWalker:
            self.baseWalker.connectChild(self)
        if connectOnInit:
            self.connect()

    def __del__(self):
        # TODO вообще-то этот код не выполняется
        OsDebug.debug('    *** OsWalker['+self.__class__.__name__+'].__del__ ***', level=0)
        #print '    *** OsWalker['+self.__class__.__name__+'].__del__ ***'
        try:
            self.disconnect()
        except:
            OsDebug.debug('    *** OsWalker['+self.__class__.__name__+'].__del__ ERROR! ***', level=0)
            #print '    *** OsWalker['+self.__class__.__name__+'].__del__ ERROR! ***'

    def setDebug(self, val):
        self.mayDebug = val

    def debug(self, val):
        if self.mayDebug:
            OsDebug.debug(val, level=0)
            #print val

    def connectCommand(self):
        return ''

    def connectExpecter(self):
        if self.connectExpecter_:
            return self.connectExpecter_
        return {
            'need':False,
            'maybe':False
        }

    def disconnectCommand(self):
        return 'exit'

    def connectChild(self, childWalker):
        self.childWalker = childWalker

    def disconnectChild(self):
        if self.childWalker:
            self.childWalker.disconnect()
            self.childWalker = False

    def connect(self, extPexpectSpawn=False):
        self.debug('    *** OsWalker['+self.__class__.__name__+'].connect ***')
        self.disconnectChild()
        disconnectCommand = self.disconnectCommand()
        command = self.command
        if not command:
            command = self.connectCommand()
        if self.baseWalker:
            self.commander = self.baseWalker.commander
            self.connection = self.baseWalker.connection
            self.doCommand(
                command,
                needExpecter=ifHasElseFalse(self.connectExpecter(),'need'),
                maybeExpecter=ifHasElseFalse(self.connectExpecter(),'maybe')
            )
        else:
            self.connection = self.makeConnection(
                command,
                disconnectCommand,
                extPexpectSpawn = extPexpectSpawn
            )
            self.commander = self.connection.getCommander()
        return self

    def makeConnection(self, command, disconnectCommand, extPexpectSpawn=False):
        # если понадобится, чтобы волкер обновременно работал только один, раскомментируй:
        # OsWalkers.connectWalker(self)
        connection = os_connect.OsConnect(
            command,
            disconnectCommand,
            ifHasElseFalse(self.connectExpecter(),'need'),
            ifHasElseFalse(self.connectExpecter(),'maybe'),
            extPexpectSpawn = extPexpectSpawn
        )
        return connection

    def disconnect(self):
        self.debug('    *** OsWalker['+self.__class__.__name__+'].disconnect ***')
        self.disconnectChild()
        if self.baseWalker:
            self.doCommand(self.disconnectCommand())
            self.connection = False
            return
        del self.connection
        try:
            del self.commander
        except:
            self.debug("    !!!!!! NO NEED TO DELETE COMMANDER")

    def isConnected(self):
        try:
            if self.connection:
                return True
        except:
            pass
        return False

    def doCommand(self, command, needExpecter=False, maybeExpecter=False):
        self.lastCommand = command
        self.clearResultMemory()
        if not self.isConnected():
            self.connect()
        else:
            self.disconnectChild()
        if not command:
            return self
        self.debug('    *** OsWalker['+self.__class__.__name__+'].doCommand: '+str(command)+' ***')
        tmp = self.commander.mayDebug
        self.commander.setDebug(self.mayDebug)
        self.commander.doCommand(
            command=command,
            needExpecter=needExpecter,
            maybeExpecter=maybeExpecter
        )
        self.commander.setDebug(tmp)
        return self

    def doPassworedCommand(self, command, password, maybeExpecter=False):
        if not maybeExpecter:
            maybeExpecter = {}
        maybeExpecter['Password:'] = password
        maybeExpecter['Пароль:'] = password
        self.doCommand(
            command = command,
            maybeExpecter = maybeExpecter
        )
        return self
        
    def clearResultMemory(self):
        OsDebug.tmpfile.memory = []
    
    def result(self):
        return '\r\n'.join(OsDebug.tmpfile.memory)
        
    def resultLines(self):
        return [a.replace('\n','') for a in self.result().split('\r\n') if len(a.replace('\n','')) and self.lastCommand != a and not '#' in a]

class SambaWalker(OsWalker):
    def __init__(self, host, shared_dir, user, password, baseWalker=False):
        self.host = host
        self.shared_dir = shared_dir
        self.user = user
        self.password = password
        super(SshWalker,self).__init__(False, baseWalker)

    def connectCommand(self):
        return ' '.join([
            'smbclient', '//'+self.host+'/'+self.shared_dir, 
            '-U', self.user, self.password
        ])

    def connectExpecter(self):
        return {
            'maybe': {
                'assword:':self.password,
                'ароль:':self.password,
                '\?':'yes'
            }
        }
        
    def putFileToHost(self, file):
        self.doCommand(
            'mput ' + file,
            maybeExpecter = {
                '?:':'y\n',
            }
        )
        
    def getFileFromHost(self, file):
        self.doCommand(
            'mget ' + file,
            maybeExpecter = {
                '?:':'y\n',
            }
        )
        
class SuWalker(OsWalker):
    def __init__(self, user, password, baseWalker=False):
        self.user = user
        self.password = password
        super(SuWalker,self).__init__(False, baseWalker)

    def connectCommand(self):
        return 'su ' + self.user

    def connectExpecter(self):
        return {
            'maybe': {
                'password:':self.password,
                'пароль:':self.password,
            }
        }

class SshWalker(OsWalker):
    def __init__(self, host, user, password, passphrase=False, baseWalker=False):
        self.host = host
        self.user = user
        self.password = password
        self.passphrase = passphrase
        super(SshWalker,self).__init__(False, baseWalker)

    def connectCommand(self):
        return 'ssh '+self.user+'@' + self.host

    def connectExpecter(self):
        dict = {
            'maybe': {
                'password:':self.password,
                'пароль:':self.password,
                '\?':'yes'
            }
        }
        if self.passphrase:
            dict['maybe']['id_rsa\':'] = self.passphrase
        return dict

class SshCopyWalker(OsWalker):
    def __init__(self, fromfile, tofile, password, passphrase=False, baseWalker=False, recursive=False):
        self.fromfile = fromfile
        self.tofile = tofile
        self.password = password
        self.passphrase = passphrase
        self.connectedCopier = False
        self.recursive = recursive
        super(SshCopyWalker,self).__init__(False, baseWalker)

    def connectCommand(self):
        if self.recursive:
            return 'scp -pr '+self.fromfile+' '+self.tofile
        return 'scp '+self.fromfile+' '+self.tofile

    def disconnectCommand(self):
        return False

    def connectExpecter(self):
        dict = {
            'need': {
                '100%':''
            },
            'maybe': {
                'password:':self.password,
                'пароль:':self.password,
                '\?':'yes'
            }
        }
        if self.passphrase:
            dict['maybe']['id_rsa\':'] = self.passphrase
        if self.recursive:
            dict['need'] = {
                '[#\$] ':False,
                'ONLY_ME':True
            }
        return dict

class SshCopier(object):
    def __init__(self, host, user, password, passphrase=False,
                 childSshCopier=False, tempDir=False):
        self.tempDir = tempDir
        self.host = host
        self.user = user
        self.password = password
        self.passphrase = passphrase
        self.childSshCopier = childSshCopier

    def getHostPath(self, file):
        return self.user+'@'+self.host+':'+file

    def getFilePath(self, file):
        if self.tempDir:
            dirs = file.split('/')
            return self.tempDir + dirs[len(dirs)-1]
        return file

    def putFileToHost(self, fromfile, tofile, baseWalker=False, recursive=False):
        if self.childSshCopier:
            topath = self.getFilePath(tofile)
        else:
            topath = tofile
        SshCopyWalker(fromfile, self.getHostPath(topath), self.password,
            passphrase=self.passphrase, baseWalker=baseWalker, recursive=recursive)
        if self.childSshCopier:
            base = SshWalker(self.host, self.user, self.password)
            self.childSshCopier.putFileToHost(topath, tofile, base, recursive=recursive)

    def getFileFromHost(self, fromfile, tofile, baseWalker=False, recursive=False):
        if self.childSshCopier:
            base = SshWalker(self.host, self.user, self.password)
            self.childSshCopier.getFileFromHost(fromfile, tofile, base, recursive=recursive)
            frompath = self.childSshCopier.getFilePath(fromfile)
        else:
            frompath = fromfile
        SshCopyWalker(self.getHostPath(frompath), self.getFilePath(tofile), self.password,
            passphrase=self.passphrase, baseWalker=baseWalker, recursive=recursive)

    def putDirToHost(self, fromdir, todir, baseWalker=False):
        self.putFileToHost(fromdir, todir, baseWalker=baseWalker, recursive=True)

    def getDirFromHost(self, fromdir, todir, baseWalker=False):
        self.getFileFromHost(fromdir, todir, baseWalker=baseWalker, recursive=True)

class RootWalker(OsWalker):
    def __init__(self, command, password):
        self.password = password
        super(RootWalker,self).__init__(command)

    def connectCommand(self):
        return 'su'

    def connectExpecter(self):
        return {
            'maybe': {
                'Password:':self.password,
                'Пароль:':self.password,
                }
        }

class PostgresWalker(OsWalker):
    def __init__(self, command, postgresUser, password):
        self.password = password
        self.postgresUser = postgresUser
        super(PostgresWalker,self).__init__(command)

    def connectCommand(self):
        return 'su ' + self.postgresUser

    def connectExpecter(self):
        return {
            'maybe': {
                'Password:':self.password,
                'Пароль:':self.password,
                }
        }
    def doPassworedCommand(self, command, maybeExpecter=False):
        return super(PostgresWalker,self).doPassworedCommand(command, self.password, maybeExpecter)
        '''
        if not maybeExpecter:
            maybeExpecter = {
                'Password:':self.password,
                'Пароль:':self.password,
            }
        self.doCommand(
            command = command,
            maybeExpecter = maybeExpecter
        )
        '''


# -*- coding: utf-8 -*-
import pexpect, sys, subprocess

class OsKeyControl(object):
    def __init__(self, key):
        self.key = key

class OsDebugWriter(object):
    def __init__(self, filename=False):
        if filename:
            self.file = open(filename, 'w')
        else:
            self.file = False
        self.printable = False

    def write(self, *args, **kwargs):
        print 'WRITE:', args
        for a in args:
            if ('ERROR' in a) or ('error' in a) or self.printable:
                print ' '.join(args)
        if self.file:
            self.file.write(*args, **kwargs)

    def flush(self, *args, **kwargs):
        print 'FLUSH:', args
        if self.file:
            self.file.flush(*args, **kwargs)
            
class OsDebugMemory(OsDebugWriter):
    def __init__(self):
        super(OsDebugMemory, self).__init__(False)
        self.memory = []

    def write(self, *args, **kwargs):
        self.memory += [args[0],]

    def flush(self, *args, **kwargs):
        #print 'FLUSH:', args
        pass
    

class OsDebug(object):
    LEVELS = {
        'EXPECTER': 0,
        'COMMANDER': 1,
        'CONNECT': 2,
    }

    debugParams = {
        'level': 8
    }

    tmpfile = False
    
    @staticmethod
    def setPrintable(val=True):
        if OsDebug.tmpfile:
            OsDebug.tmpfile.printable = val

    @staticmethod
    def connectToTmpFile():
        OsDebug.tmpfile = OsDebugWriter('/tmp/os_debug.tmp')
        
    @staticmethod
    def connectToNulFile():
        OsDebug.tmpfile = OsDebugWriter(False)
        
    @staticmethod
    def connectToMemory():
        OsDebug.tmpfile = OsDebugMemory()

    @staticmethod
    def logfile():
        if OsDebug.tmpfile:
            return OsDebug.tmpfile
        return sys.stdout

    @staticmethod
    def setLevel(level):
        OsDebug.debugParams['level'] = level
        
    @staticmethod
    def getLevel():
        return OsDebug.debugParams['level']

    @staticmethod
    def debug(*args, **kwargs):
        if 'level' in kwargs:
            level = kwargs['level']
        else:
            level = 0
        if level >= OsDebug.debugParams['level']:
            print ' '.join([str(a) for a in args])

class QuestsAnswersObject(object):
    def __init__(self, mass):
        if not mass:
            self.quests = []
            self.answers = []
            self.needOnlyMe = False
            return
        self.quests = []
        self.answers = []
        self.needOnlyMe = False
        for quest in mass.keys():
            if quest == 'ONLY_ME':
                if mass[quest]:
                    self.needOnlyMe = True
            else:
                self.quests += [quest]
                self.answers += [mass[quest]]
            
'''
def getQuestsAndAnswers(mass):
    if not mass:
        return [], [], False
    quests = []
    answers = []
    needOnlyMe = False
    for quest in mass.keys():
        if quest == 'ONLY_ME':
            if mass[quest]:
                needOnlyMe = True
        else:
            quests += [quest]
            answers += [mass[quest]]
    return quests, answers, needOnlyMe
'''
    
class NeedTester(object):
    def __init__(self, needObject = False, maybeObject = False):
        self.needObject = needObject
        self.maybeObject = maybeObject
        count = len(needObject.quests)
        self.needBoolStart = 1 + len(maybeObject.quests)
        self.massive = []
        self.fin = False
        if count:
            k = count
            while k > 0:
                self.massive += [False,]
                k -= 1
            self.needMe = True
            self.appendNeedToMaybe()
        else:
            self.needMe = False
            
    def appendNeedToMaybe(self):
        self.maybeObject.quests += self.needObject.quests
        self.maybeObject.answers += self.needObject.answers
            
    def setChecked(self, i):
        self.massive[i] = True
    def isFilled(self):
        for m in self.massive:
            if not m:
                return False
        return True
    def setCheckedByExpect(i):
        pos = i - self.needBoolStart
        if pos >= 0:
            self.setChecked(pos)
    def execute(self, i):
        if self.needMe:
            self.setCheckedByExpect(i)
            if self.isFilled():
                if self.needObject.needOnlyMe:
                    self.fin = True
                    return self
                self.needMe = False
        if i == 0:
            self.fin = True
        return self
        

class OsExpecter(object):
    def __init__(self, child, commander=False):
        self.expecter = "[#\$] "
        self.child = child
        self.commander = commander
        if commander and not commander.expecter:
            commander.expecter = self
        self.mayDebug = True

    def setDebug(self, val):
        self.mayDebug = val

    def doExpect(self, exp):
        self.debugIfMay('    +++ expect start:', str(exp), '+++', level=0)
        i = self.child.expect(exp, timeout=None)
        self.debugIfMay('\n    +++ expect finded:', str(exp[i]), '+++', level=0)
        return i
        
    def debugIfMay(self, *args, **kwargs):
        if self.mayDebug:
            OsDebug.debug(*args, **kwargs)

    def expect(self, needExpecter=False, maybeExpecter=False):
        needTester = NeedTester(
            needObject = QuestsAnswersObject(needExpecter),
            maybeObject = QuestsAnswersObject(maybeExpecter)
        )
        self.expectWhile(needTester)
        
    def expectWhile(self, needTester):
        while True:
            i = self.doExpect([self.expecter,] + needTester.maybeObject.quests)
            if needTester.execute(i).fin:
                return
            answer = needTester.maybeObject.answers[i-1]
            if answer:
                self.commander.sendCommand(answer)
            else:
                self.tryExpect(needTester.needObject.quests)
                return

    def tryExpect(self, needQuests=[]):
        try:
            exp = self.expecter
            i = self.doExpect([exp,] + needQuests)
        except:
            pass

class OsCommander(object):
    def __init__(self, child, expecter=False):
        self.expecter = expecter
        if expecter and not expecter.commander:
            expecter.commander = self
        self.child = child
        self.mayDebug = True

    def setDebug(self, val):
        self.mayDebug = val
        self.expecter.setDebug(val)
        
    def debugIfMay(self, *args, **kwargs):
        if self.mayDebug:
            OsDebug.debug(*args, **kwargs)

    '''
    def sendCommand(self, command):
        self.debugIfMay("    *** OsCommander.sendCommand:", command, "***", level=0)
        self.child.sendline(command)
    '''
        
    def sendCommand(self, command):
        self.debugIfMay("    *** OsCommander.sendCommand:", command, "***", level=0)
        if command.__class__.__name__ == 'OsKeyControl':
            self.child.sendcontrol(command.key)
        else:
            self.child.sendline(command)

    def doCommand(self, command, needExpecter=False, maybeExpecter=False):
        self.debugIfMay("    *** OsCommander.doCommand:", command, "***", level=0)
        if command:
            self.sendCommand(command)
        self.expecter.expect(needExpecter, maybeExpecter)

class OsConnect(object):
    def __init__(self, command, disconnectCommand, needExpecter, maybeExpecter, extPexpectSpawn=False):
        self.disconnectCommand = disconnectCommand
        self.connect(
            command, needExpecter, maybeExpecter, extPexpectSpawn
        )

    def __del__(self):
        self.disconnect()

    def getCommander(self):
        return self.osCommander

    def connect(self, command, needExpecter, maybeExpecter, extPexpectSpawn=False):
        if extPexpectSpawn:
            child = self.sendConnect(command, pexpectSpawn=extPexpectSpawn)
        else:
            child = self.sendConnect(command)
        self.osCommander = OsCommander(child, False)
        expecter = OsExpecter(child, self.osCommander)
        self.osCommander.expecter = expecter
        expecter.expect(needExpecter, maybeExpecter)
        return child, expecter

    def sendConnect(self, command, pexpectSpawn=pexpect.spawn):
        OsDebug.debug("    *** OsConnectPrivate.sendConnect:", command, "***", level=0)
        self.child = pexpectSpawn(command)
        self.child.logfile = OsDebug.logfile()
        return self.child

    def disconnect(self):
        OsDebug.debug("    *** OsConnectPrivate.disconnect ***", level=0)
        self.osCommander.doCommand(
            self.disconnectCommand,
            maybeExpecter={pexpect.EOF:False}
        )




# setup.py
#try:
#    from setuptools import setup
#except:
#    from distutils.core import setup
    
from setuptools import setup, find_packages
from os.path import join, dirname

import osconnect

setup(
    name = 'osconnect',
    version = osconnect.__version__,
    packages = find_packages(),
    long_description = open(join(dirname(__file__), 'README.txt')).read(),
    install_requires=[
        'pexpect',#==0.8',
    ],
    test_suite='tests',
)


# -*- coding: utf-8 -*-
import pexpect, sys, subprocess

def getSubprocess(command, **params):
    return subprocess.Popen(command, shell=True, stdout=subprocess.PIPE, **params)

def simpleCommand(command, **params):
    return getSubprocess(command, **params).stdout.read()
    
def simpleCommandLines(command, **params):
    return getSubprocess(command, **params).stdout.readlines()

def tryCommand(command, **params):
    try:
        subprocess.check_call(command, shell=True, stdout=subprocess.PIPE, **params)
        #lines = getSubprocess(command, **params).stdout.readlines()
        return True
    except:
        return False